/*
    Umlaut converter, CGI version.
    (See also \targon\twl\umlaut(

    Copyright 2000, R.Harmsen

    5 Dec 2014: Added meta viewport and hyperlinks.

    16 September 2022: Measures against Cross-Site Scripting (XSS).
	See function against_XSS, and see:
	https://www.openbugbounty.org/
	https://cwe.mitre.org/data/definitions/79.html
	https://cheatsheetseries.owasp.org/cheatsheets/Cross_Site_Scripting_Prevention_Cheat_Sheet.html
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <math.h>

#if defined(_Windows) || defined (__MSDOS__)
   #include "io.h"
   #include "fcntl.h"
#endif

#include "ae-acc.h"
#include "../cgitools/cgitools.h"


enum algor_enum
{
   ALGOR_UE_U,
   ALGOR_U_UE,
};
static char *AlgorText[] =
{
   "ue --> \xfc",
   "\xfc --> ue",
};

static char inittext[] =
{
   "Hier der zu konvertierende Text.\n"
   "Zuerst ein Paar Beispielwrter:\n"
   "berhaupt, nderung, ndern und lschen, etwas neues,\n"
   "teuer. Buenos Aires ist natrlich ein spanischer Name.\n"

   "Die Konversion luft noch nach der alten Rechtschreibung, \n"
   /* Korrektur 24. April 2015: Na => na */
   "so da es Fa ist, und na, und Ha.\n"
   "Echte Fehler gibt es auch noch:\n"
   "Poet, Poesie, die Nsse, er sa (sie se), die Masse/Mae;\n"
   "das Ma, in hohem Mae, ich mu, ich mute oder mte,\n"
   "ich wute, es bi, sie ri, wie ihr wit.\n"
};

void showform (char *script, char *input, enum algor_enum algor);
void htmlhead (void);
void htmlfoot (void);
void getparms (int len, char *input, enum algor_enum *p_algor);
int calculate (int len, char *input, enum algor_enum algor);
void against_XSS (char *input);

#if 0
static int isletterofword (char *p);
#endif

/***********************************************************************

 **********************************************************************/
int main (void)
{
   int len;
   #if 1
      char *scriptname = "../cgi-bin/umlaut.cgi";
   #else
      #if defined(_Windows) || defined (__MSDOS__)
         char *scriptname = "//127.0.0.1/cgi-bin/umlaut.cgi";
      #else
         char *scriptname = "//rudhar.com/cgi-bin/umlaut.cgi";
      #endif
   #endif
   enum algor_enum algor = ALGOR_U_UE;
   char *input;

   input = calloc(1, sizeof inittext);
   strcpy(input, inittext);

   #if defined(_Windows) || defined (__MSDOS__)
      setmode(fileno(stdin ), O_BINARY);
      setmode(fileno(stdout), O_BINARY);
   #endif

   if ((len = userinput()) == 0)
   {
      htmlhead();
      showform(scriptname, input, algor);
      calculate(sizeof inittext, input, algor);
      htmlfoot();
   }
   else
   {
      input = realloc(input, len);
      getparms(len, input, &algor);
      htmlhead();
      showform(scriptname, input, algor);
      calculate(len, input, algor);
      htmlfoot();
   }
   free(input);

   return 0;
}


/***********************************************************************

 **********************************************************************/
void showform (char *script, char *input, enum algor_enum algor)
{
   against_XSS(input);

   printf(
      "<p><h1>Automatische Umlautumwandlung</h1></p>\n"
      "<p>\n"
      "<FORM ACTION=\"%s\" METHOD=POST>\n"
      "<TEXTAREA NAME=Intext ROWS=11 COLS=80>%s</TEXTAREA>\n"
      "<br><br>\n"
      "<INPUT TYPE=Submit NAME=todo1 VALUE=\"%s\">\n"
      "<INPUT TYPE=Submit NAME=todo2 VALUE=\"%s\">\n"
      "</FORM></p>\n",
      script, input,
      AlgorText[0],
      AlgorText[1]);
}


/***********************************************************************

 **********************************************************************/
void htmlhead (void)
{
   printf(
      "Content-type: text/html\n\n"
      "<!doctype html>\n"
      "<html>\n"
      "<head>\n"
      "<title>Automatische Umlautkonversion</title>\n"
      "<link rel=stylesheet href=\"../css/cpyrport.css\">\n"
      "<link rel=stylesheet href=\"../cgi-bin/colschem.cgi\">\n"
      "<style>body {margin: 25px;}</style>\n"
      "<meta name=viewport content=\"width=device-width, initial-scale=1\">\n"
      "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=ISO-8859-1\">\n"
      "</head>\n\n"
      "<body>\n"
      );
   printf(
      "<p class=NCol>\n"
      "<a class=NCol href=\"../index/colexpnl.htm\">Farben:</a>\n"
      "<a class=NCol href=\"../cgi-bin/colcooky.cgi?NoWeirdColours=1\">Neutral</a>\n"
      "<a class=NCol href=\"../cgi-bin/colcooky.cgi?NoWeirdColours=0\">Eigensinnig</a>\n"
      "<a class=NCol href=\"javascript:location.reload();\">Neu laden</a>\n"
      "</p>"
      );
   printf(
      "<p class=pagelink>\n"
      "<a href=\"../lingtics/lingtics.htm\">Sprachenmen</a>\n"
      "<a href=\"../index/menu.htm\">Hauptmen</a>\n"
      "</p>"
      );

   fflush(stdout);
}

/***********************************************************************

 **********************************************************************/
void htmlfoot (void)
{
   printf("</BODY></HTML>\n");
   fflush(stdout);
}

/***********************************************************************

 ****************\******************************************************/
void getparms (int len, char *input, enum algor_enum *p_algor)
{
   char *eq1, *eq2;
   char *amp;
   char *buf = calloc(len + 1, 1);

   if (!buf ||
      fread(buf, len, 1, stdin) != 1)
   {
      free(buf);
      return;
   }

   eq1 = strchr(buf, '=');
   if (!eq1)
   {
      free(buf);
      return;
   }

   eq1++;
   if ((amp = strchr(eq1, '&')) != NULL)
   {
      *amp = '\0';
   }
   memset(input, '\0', len);
   strncpy(input, eq1, len - 1);
   decode_cgi_string(input);
   if (strlen(input) == 0)
   {
      strcpy(input, inittext);
   }

   if (amp)
      eq1 = amp + 1;

   eq2 = strchr(eq1, '=');
   if (!eq2)
   {
      free(buf);
      return;
   }

   eq2++;
   if ((amp = strchr(eq2, '&')) != NULL)
   {
      *amp = '\0';
   }
   decode_cgi_string(eq2);

   if (strncmp(eq2, AlgorText[0], strlen(AlgorText[0])) == 0)
      *p_algor = ALGOR_UE_U;
   else if (strncmp(eq2, AlgorText[1], strlen(AlgorText[1])) == 0)
      *p_algor = ALGOR_U_UE;

   free(buf);
}


/***********************************************************************

 **********************************************************************/
int calculate (int len, char *input, enum algor_enum algor)
{
   char *out;
   char *tus;
   const char ruler[] = "<hr width=98%% align=center>";

   out = malloc(len + len / 2);
   if (!out)
      printf("Alloc error 1\n");

   switch (algor)
   {
      case ALGOR_UE_U:
         ae_to_acc (input);
         acc_to_iso(out, input);

         printf(ruler);
         printf("<pre>");
         printf("%s", out);
         printf("</pre>");
         printf(ruler);
         break;

      case ALGOR_U_UE:
      default:
         printf(ruler);
         printf("Konvertiert:");
         printf("<pre>");
         acc_to_ae(out, input);
         printf("%s", out);
         printf ("</pre>");

         tus = malloc(len + len / 2);
         if (tus)
         {
            strcpy(tus, out);
            ae_to_acc(tus);
            acc_to_iso(out, tus);

            printf(ruler);
            printf("Zur&uuml;ckkonvertiert:");
            printf("<pre>");
            printf("%s", out);
            printf("</pre>");

            printf(ruler);
            printf("Unterschiede:");
            printf("<pre>");
            repdiff(input, out);
            printf("</pre>");

            free(tus);
         }
         printf(ruler);
         break;
   }
   free(out);


   return 0;
}


/***********************************************************************

 **********************************************************************/
/* Niet gebruikt. misschien later nog. */
#if 0
static int isletterofword (char *p)
{
   if (isascii(*p))
   {
      if (islower(*p) || isupper(*p) || isdigit(*p))
          return 1;
   }
   else
   {
      if (*p == '\xe4' /* a-umlaut   */ ||
          *p == '\xf6' /* o-umlaut   */ ||
          *p == '\xfc' /* u-umlaut   */ ||
          *p == '\xc4' /* A-umlaut   */ ||
          *p == '\xd6' /* O-umlaut   */ ||
          *p == '\xdc' /* U-umlaut   */ ||
          *p == '\xdf' /* Scharfes s */)
      {
         return 1;
      }
   }
   return 0;
}
#endif

/***********************************************************************
   Wipe out any < and > in input entered by the user,
   to avoid maliciously input HTML code, and possible
   scripting languages within that.
 **********************************************************************/
static char to_erase[] = "<>";
void against_XSS (char *input)
{
   char *p, *q;

   for (p = input; *p; p++)
      for (q = to_erase; *q; q++)
         if (*p == *q) *p = ' ';
}

